# Generated by Django 6.0.1 on 2026-02-24 14:24

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("fileindex", "0005_use_bigautofield_for_ids"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="indexedfile",
            index=models.Index(fields=["derived_from", "mime_type"], name="fileindex_i_derived_136784_idx"),
        ),
    ]
