# Generated by Django 5.2.5 on 2025-08-17 19:15

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("fileindex", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="filepath",
            name="id",
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="indexedfile",
            name="id",
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
    ]
